/*-------------------------------
    gamelist.h - nate wallace
	Contains functions for loading game data and
	preferences from file.
	--------------------------------*/

#include <windows.h>
#include <stdio.h>
#include "jpeglib.h"
#include <setjmp.h>

/* define tag elements */
#define GAMETAG TEXT("<GAME>")
#define _GAMETAG TEXT("</GAME>")
#define NAMETAG TEXT("<NAME>")
#define _NAMETAG TEXT("</NAME>")
#define PATHTAG TEXT("<PATH>")
#define _PATHTAG TEXT("</PATH>")
#define PICTAG TEXT("<PIC>")
#define _PICTAG TEXT("</PIC>")
#define DESCTAG TEXT("<DESC>")
#define _DESCTAG TEXT("</DESC>")
#define ALPHATAG TEXT("<ALPHA>")
#define _ALPHATAG TEXT("</ALPHA>")
#define MENUTAG TEXT("<MENU>")
#define _MENUTAG TEXT("</MENU>")
#define BORDERTAG TEXT("<BORDER>")
#define _BORDERTAG TEXT("</BORDER>")
#define SCROLLTAG TEXT("<SCROLL>")
#define _SCROLLTAG TEXT("</SCROLL>")
#define FONTNAMETAG TEXT("<FONTNAME>")
#define _FONTNAMETAG TEXT("</FONTNAME>")
#define FONTCOLORTAG TEXT("<FONTCOLOR>")
#define _FONTCOLORTAG TEXT("</FONTCOLOR>")
#define FONTSIZETAG TEXT("<FONTSIZE>")
#define _FONTSIZETAG TEXT("</FONTSIZE>")
#define BACKGROUNDTAG TEXT("<BACKGROUND>")
#define _BACKGROUNDTAG TEXT("</BACKGROUND>")
#define SELECTCOLORTAG TEXT("<SELECT>")
#define _SELECTCOLORTAG TEXT("</SELECT>")

/* game data */
struct game {
	char name[100];
	char path[MAX_PATH];
	char pic[MAX_PATH];
	char desc[1000];
};
typedef struct game GAME;

/* list of games */
struct gamelist {
	struct gamelist *next;
	struct gamelist *prev;
	GAME node;
};
typedef struct gamelist GAMELIST;

GAMELIST *gamehead;			/* main list for games */
GAMELIST *currentgame;		/* used to iterate through list */

char background[1000];
char fontname[32];
int fontsize;

float alpha;
int borderR, borderG, borderB;	/* border color for windows */
int menuR, menuG, menuB;		/* background color for windows */
int scrollR, scrollG, scrollB;  /* color of scroll bar */
int fontR, fontG, fontB;
int selectR, selectG, selectB;

int gamecount = 0;

char* GetProgramStr( const char *path ) {
	char *block, *program, *tmp;
	program = NULL;

	block = strrchr( path, '/' ) + 1;
	if( block ) { 
		/* eat leading whitespace */
		while( block[0] == ' ' && block[0] != NULL )
			block = &block[1];
 
		/* seperate program from parameters */
		tmp = strchr( block, ' ' );
		if( tmp ) {
			program = malloc( strlen(path) - strlen(tmp) + 1 );
			program[0] = NULL;
			strncat( program, path, strlen(path) - strlen(tmp) );
		}	
		else {
			program = malloc( strlen(path) + 1 );
			strcpy( program, path );
		}
	}
	return program;
}

char* GetDirectoryStr( const char *path ) {
	char *directory, *tmp;
	directory = NULL;

	tmp = strrchr( path, '/' );
	if( tmp ) {
		directory = malloc( strlen(path) - strlen(tmp) + 2 );
		directory[0] = NULL;
		strncat( directory, path, strlen(path) - strlen(tmp) + 1 );
	}
	return directory;
}

char* GetParamStr( const char *path ) {
	char *block, *param;
	param = NULL;

	block = strrchr( path, '/' ) + 1;
	if( block ) { 
		/* eat leading whitespace */
		while( block[0] == ' ' && block[0] != NULL )
			block = &block[1];

		block = strchr( block, ' ' );
		if( block ) {
			param = malloc( strlen(block) + 1 );
			param[0] = NULL;
			strcpy( param, block );
		}
	}
	return param;
}

int Load( char *filename ) {

	DWORD dwFileSize, dwHighSize, dwBytesRead;
	HANDLE hFile;
	char *buffer, *strHead, *temp;
	char *block;
	char rgb[1000];
	char gameBlock[1000];
	int i, test;

	/* default values */
	alpha = .5;
	borderR = borderG = 0;
	borderB = 155;
	menuR = menuG = 0;
	menuB = 255;
	scrollR = scrollG = 0;
	scrollB = 255;
	selectR = 240; 
	selectG = 255;
	selectB = 0;
	fontR = fontG = fontB = 0;
	fontsize = 16;

	/* load and read file */
	hFile = CreateFile( filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL );

	if( hFile == INVALID_HANDLE_VALUE )
		return 1;

	dwFileSize = GetFileSize( hFile, &dwHighSize );

	if( dwHighSize ) {
		CloseHandle( hFile );
		return 1;
	}

	buffer = malloc( dwFileSize );

	if( !buffer ) {
		CloseHandle( hFile );
		return 1;
	}

	ReadFile( hFile, buffer, dwFileSize, &dwBytesRead, NULL );
	CloseHandle( hFile );
	temp = buffer;
	rgb[0] = NULL;

	/* convert all tags to uppercase */
	test = 0;
	for( i = 0; i < strlen(buffer); i++ ) {
		if( test )
			buffer[i] = toupper( buffer[i] );
		if( buffer[i] == '<' )
			test = 1;
		if( buffer[i] == '>' )
			test = 0;
	}
	
	/* parse background path */
	if( strHead = strstr( buffer, BACKGROUNDTAG ) ) {
		buffer = strHead + strlen(BACKGROUNDTAG) * sizeof(char);
		if( strstr( buffer, _BACKGROUNDTAG ) ) 
			strncat( background, buffer, strlen(buffer) - strlen( strstr(buffer, _BACKGROUNDTAG) ) );

		buffer = temp;
	}

	/* parse font name */
	if( strHead = strstr( buffer, FONTNAMETAG ) ) {
		buffer = strHead + strlen(FONTNAMETAG) * sizeof(char);
		if( strstr( buffer, _FONTNAMETAG ) ) 
			strncat( fontname, buffer, strlen(buffer) - strlen( strstr(buffer, _FONTNAMETAG) ) );

		buffer = temp;
	}

	/* get alpha */
	if( strHead = strstr( buffer, ALPHATAG ) ) {
		buffer = strHead + strlen(ALPHATAG) * sizeof(char);
		if( strstr( buffer, _ALPHATAG ) ) {
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _ALPHATAG) ) );
			alpha = atof( rgb );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get menu rgb */
	if( strHead = strstr( buffer, MENUTAG ) ) {
		buffer = strHead + strlen(MENUTAG) * sizeof(char);
		if( strstr( buffer, _MENUTAG ) ) {
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _MENUTAG) ) );

			menuR = atoi( rgb );
			menuG = atoi( strchr(rgb,',') + sizeof(char) );
			menuB = atoi( strrchr(rgb,',') + sizeof(char) );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get border rgb */
	if( strHead = strstr( buffer, BORDERTAG ) ) {
		buffer = strHead + strlen(BORDERTAG) * sizeof(char);
		if( strstr( buffer, _BORDERTAG ) ) {
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _BORDERTAG) ) );

			borderR = atoi( rgb );
			borderG = atoi( strchr(rgb,',') + sizeof(char) );
			borderB = atoi( strrchr(rgb,',') + sizeof(char) );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get scroll rgb */
	if( strHead = strstr( buffer, SCROLLTAG ) ) {
		buffer = strHead + strlen(SCROLLTAG) * sizeof(char);
		if( strstr( buffer, _SCROLLTAG ) ) {
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _SCROLLTAG) ) );

			scrollR = atoi( rgb );
			scrollG = atoi( strchr(rgb,',') + sizeof(char) );
			scrollB = atoi( strrchr(rgb,',') + sizeof(char) );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get font rgb */
	if( strHead = strstr( buffer, FONTCOLORTAG ) ) {
		buffer = strHead + strlen(FONTCOLORTAG) * sizeof(char);
		if( strstr( buffer, _FONTCOLORTAG ) ) { 
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _FONTCOLORTAG) ) );

			fontR = atoi( rgb );
			fontG = atoi( strchr(rgb,',') + sizeof(char) );
			fontB = atoi( strrchr(rgb,',') + sizeof(char) );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get select frame rgb */
	if( strHead = strstr( buffer, SELECTCOLORTAG ) ) {
		buffer = strHead + strlen(SELECTCOLORTAG) * sizeof(char);
		if( strstr( buffer, _SELECTCOLORTAG ) ) {
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _SELECTCOLORTAG) ) );

			selectR = atoi( rgb );
			selectG = atoi( strchr(rgb,',') + sizeof(char) );
			selectB = atoi( strrchr(rgb,',') + sizeof(char) );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* get font size */
	if( strHead = strstr( buffer, FONTSIZETAG ) ) {
		buffer = strHead + strlen(FONTSIZETAG) * sizeof(char);
		if( strstr( buffer, _FONTSIZETAG ) ) { 
			strncat( rgb, buffer, strlen(buffer) - strlen( strstr(buffer, _FONTSIZETAG) ) );
			fontsize = atoi( rgb );
		}

		rgb[0] = NULL;
		buffer = temp;
	}

	/* parse game info */
	while( strHead = strstr( buffer, GAMETAG ) ) {
		gamecount++;

		/* create space in memory */
		if( !currentgame ) {
			currentgame = malloc( sizeof(GAMELIST) );
			currentgame->next = NULL;
			currentgame->prev = NULL;
			gamehead = currentgame;
		}
		else {
			currentgame->next = malloc( sizeof(GAMELIST) );
			currentgame->next->prev = currentgame;
			currentgame = currentgame->next;
			currentgame->next = NULL;
		}
		/* defaults */
		currentgame->node.name[0] = NULL;
		currentgame->node.path[0] = NULL;
		currentgame->node.pic[0] = NULL;
		currentgame->node.desc[0] = NULL;
	
		/* take info from between tags */
		buffer = strHead + strlen(GAMETAG) * sizeof(char);
		block = NULL;
		gameBlock[0] = NULL;

		if( strstr( buffer, _GAMETAG ) ) {
			strncat( gameBlock, buffer, strlen(buffer) - strlen( strstr(buffer, _GAMETAG) ) );

		/* get name */
			if( strstr( gameBlock, NAMETAG ) ) {
				block = strstr( gameBlock, NAMETAG ) + strlen( NAMETAG ) * sizeof(char);
				if( strstr( block, _PATHTAG ) )
					strncat( currentgame->node.name, block, strlen(block) - strlen( strstr(block, _NAMETAG) ) );
				block = NULL;
			}

		/* get path */
			if( strstr( gameBlock, PATHTAG ) ) {
				block = strstr( gameBlock, PATHTAG ) + strlen( PATHTAG ) * sizeof(char); 
				if( strstr( block, _PATHTAG ) ) 
					strncat( currentgame->node.path, block, strlen(block) - strlen( strstr(block, _PATHTAG) ) );
				block = NULL;
			}

		/* get pic */
			if( strstr( gameBlock, PICTAG ) ) {
				block = strstr( gameBlock, PICTAG ) + strlen( PICTAG ) * sizeof(char);
				if( strstr( block, _PICTAG ) )
					strncat( currentgame->node.pic, block, strlen(block) - strlen( strstr(block, _PICTAG) ) );
				block = NULL;
			}

		/* get desc */
			if( strstr( gameBlock, DESCTAG ) ) {
				block = strstr( gameBlock, DESCTAG ) + strlen( DESCTAG ) * sizeof(char);
				if( strstr( block, _DESCTAG ) )
					strncat( currentgame->node.desc, block, strlen(block) - strlen( strstr(block, _DESCTAG) ) );
				block = NULL;
			}
		}
	}

	free( temp );

	return 0;
}

/* 
 *	This section of code is taken from the IJG website: www.ijg.org
 *  This code works with version 6b of their freely distributed library
 *  Also note that jmorecfg.h had to be modified for it to work properly 
 *  here.
 */
struct my_error_mgr {
  struct jpeg_error_mgr pub;	/* "public" fields */

  jmp_buf setjmp_buffer;	/* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;

/*
 * Here's the routine that will replace the standard error_exit method:
 */

METHODDEF(void)
my_error_exit (j_common_ptr cinfo)
{
  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  my_error_ptr myerr = (my_error_ptr) cinfo->err;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message) (cinfo);

  /* Return control to the setjmp point */
  longjmp(myerr->setjmp_buffer, 1);
}

int read_JPEG_file ( char * filename)
{
  HDC hdc;
  HBITMAP hBitmap;
  int pad;
  byte *bits;
  BITMAPINFO dibInfo;

  struct jpeg_decompress_struct cinfo;

  struct my_error_mgr jerr;
  /* More stuff */
  FILE * infile;		/* source file */
  JSAMPARRAY buffer;	/* Output row buffer */
  int row_stride;		/* physical row width in output buffer */

  if ((infile = fopen(filename, "rb")) == NULL) {
    return NULL;
  }

  /* Step 1: allocate and initialize JPEG decompression object */
  cinfo.err = jpeg_std_error(&jerr.pub);
  jerr.pub.error_exit = my_error_exit;
  /* Establish the setjmp return context for my_error_exit to use. */
  if (setjmp(jerr.setjmp_buffer)) {
    /* If we get here, the JPEG code has signaled an error.
     * We need to clean up the JPEG object, close the input file, and return.
     */
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
    return 0;
  }
  jpeg_create_decompress(&cinfo);

  /* Step 2: specify data source (eg, a file) */
  jpeg_stdio_src(&cinfo, infile);

  /* Step 3: read file parameters with jpeg_read_header() */
  (void) jpeg_read_header(&cinfo, TRUE);

  /* Step 4: set parameters for decompression */
  /* In this example, we don't need to change any of the defaults set by
   * jpeg_read_header(), so we do nothing here.
   */

  /* Step 5: Start decompressor */
  (void) jpeg_start_decompress(&cinfo);

  row_stride = cinfo.output_width * cinfo.output_components; 

  buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);
  /* Step 6: while (scan lines remain to be read) */
  /*           jpeg_read_scanlines(...); */
  hdc = GetDC(NULL);
  	dibInfo.bmiHeader.biBitCount = 24;
	dibInfo.bmiHeader.biClrImportant = 0;
	dibInfo.bmiHeader.biClrUsed = 0;
	dibInfo.bmiHeader.biCompression = 0;
	dibInfo.bmiHeader.biHeight = cinfo.output_height;
	dibInfo.bmiHeader.biPlanes = 1;
	dibInfo.bmiHeader.biSize = 40;
	dibInfo.bmiHeader.biSizeImage = row_stride * cinfo.output_height;
	dibInfo.bmiHeader.biWidth = cinfo.output_width;
	dibInfo.bmiHeader.biXPelsPerMeter = 2835;
	dibInfo.bmiHeader.biYPelsPerMeter = 2835;
	dibInfo.bmiColors[0].rgbBlue = 0;
	dibInfo.bmiColors[0].rgbGreen = 0;
	dibInfo.bmiColors[0].rgbRed = 0;
	dibInfo.bmiColors[0].rgbReserved = 0;
  ReleaseDC(NULL,hdc);

	hBitmap = CreateDIBSection( hdc, &dibInfo, DIB_RGB_COLORS, &bits, NULL, 0 );
	pad = (int)4 * ((cinfo.output_width * 24 + 31) / 32) - row_stride;

  while (cinfo.output_scanline < cinfo.output_height) {
	jpeg_read_scanlines(&cinfo, buffer, 1); 
	memcpy( bits + (row_stride+pad) * (cinfo.output_height - cinfo.output_scanline), buffer[0], row_stride );
  } 

		
  /* Step 7: Finish decompression */
  (void) jpeg_finish_decompress(&cinfo);

  /* Step 8: Release JPEG decompression object */
  jpeg_destroy_decompress(&cinfo);
  fclose(infile);


  return hBitmap;
}


HBITMAP LoadAnImage(char* filefame) {


	return NULL;
}

/* free memory in list */
int CleanUp()
{
	GAMELIST *next;
	currentgame = gamehead;

	while( currentgame )
	{
		next = currentgame->next;
		free( currentgame );
		currentgame = next;
	}

	return 0;
}