/*------------------------------------------------------------
   GameLoader.c - nate wallace
   main file for project.  
  ------------------------------------------------------------*/

#include <windows.h>
#include "resource.h"
#include "gamelist.h"

#define DESCWINDOW 101
#define LISTWINDOW 102

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static TCHAR szAppName[] = TEXT ("GameLoader");
     HWND         hwnd ;
     MSG          msg ;
     WNDCLASS     wndclass;

     wndclass.style         = CS_HREDRAW | CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon( hInstance, MAKEINTRESOURCE(IDI_ICON1) );
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = CreateSolidBrush(RGB(255,0,0));
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;

     RegisterClass (&wndclass);

     hwnd = CreateWindow(  
					       szAppName,                  
                           NULL,						  
                           WS_POPUP | WS_CLIPCHILDREN,	      /* run in fullscreen */				  
                           CW_USEDEFAULT,              
                           CW_USEDEFAULT,              
                           CW_USEDEFAULT,              
                           CW_USEDEFAULT,              
                           NULL,                       
                           NULL,                       
                           hInstance,                  
                           NULL) ;                     
							  

	 ShowCursor(FALSE); /* no cursor */
     ShowWindow (hwnd, SW_MAXIMIZE);
     UpdateWindow (hwnd);
     
     while (GetMessage (&msg, NULL, 0, 0))
     {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
     }
     return msg.wParam ;
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static int xClient, yClient;		/* screen size */
	static int yListTop, yDescTop;		/* text positioning */
	static int focus;					/* scrolling */
	int x, y;

	/* bitmaps */
	static HBITMAP hBackground;
	static HBITMAP hPicBits;
	HBITMAP hTempBits;

	/* windows */
	static RECT titleRect;
	static RECT listRect;
	static RECT picRect;
	static RECT descRect;
	static RECT captionRect;

	static TEXTMETRIC tm;
	HWND splash;

	/* alpha */
	int R, G, B;
	COLORREF pixel;

	/* painting */
	HDC hdc, hdcMem, hdcImg; 
	BITMAP bitmap;
	RECT rect;

	static HFONT font;
	static GAMELIST *selectedGame;	/* current game selected */

	switch (message) {
	 
	case WM_CREATE:
		xClient = GetSystemMetrics(SM_CXSCREEN);
		yClient = GetSystemMetrics(SM_CYSCREEN);

		/* splash screen */
		hTempBits = read_JPEG_file("img/splash.jpg");
		splash = CreateWindow( TEXT("static"), NULL, WS_POPUP | WS_VISIBLE, 
							   0, 0, xClient, yClient, NULL, (HMENU)1000,
							   ((LPCREATESTRUCT)lParam)->hInstance, NULL );

		hdc = GetDC( splash );
		GetObject( hTempBits, sizeof(BITMAP), &bitmap );

		hdcImg = CreateCompatibleDC( hdc );
		SelectObject( hdcImg, hTempBits );

		BitBlt( hdc, (xClient - bitmap.bmWidth)/2, (yClient - bitmap.bmHeight)/2, bitmap.bmWidth, bitmap.bmHeight, hdcImg, 0, 0, SRCCOPY );

		ShowWindow( splash, SW_SHOW );
		DeleteObject( hTempBits );
		DeleteDC( hdcImg );
		ReleaseDC( splash, hdc );

		/* load and setup game info */
		Load( TEXT( "config.cfg" ) );
		selectedGame = gamehead;
		focus = LISTWINDOW;

		/* set screen regions */
		SetRect( &titleRect, xClient*4/60, yClient*2/45, xClient*56/60, yClient*4/45 );
		SetRect( &listRect, xClient*4/60, yClient*6/45, xClient*29/60, yClient*24/45 );
		SetRect( &picRect, xClient*30/60, yClient*6/45, xClient*56/60, yClient*24/45 );
		SetRect( &descRect, xClient*4/60, yClient*25/45, xClient*56/60, yClient*38/45 );
		SetRect( &captionRect, xClient*4/60, yClient*40/45, xClient*56/60, yClient*42/45 );

		/* load background image from file and scale it */
		hBackground = read_JPEG_file ( background );

		hdc = GetDC( hwnd );

		hdcMem = CreateCompatibleDC( hdc );
		hdcImg = CreateCompatibleDC( hdc );

		hTempBits = CreateCompatibleBitmap( hdc, xClient, yClient );
		SelectObject( hdcMem, hTempBits );
		SelectObject( hdcImg, hBackground );

		SetStretchBltMode( hdcMem, HALFTONE );
		GetObject( hBackground, sizeof( BITMAP ), &bitmap );
		StretchBlt( hdcMem, 0, 0, xClient, yClient, hdcImg, 0, 0, bitmap.bmWidth, bitmap.bmHeight, MERGECOPY );

		DeleteObject( hBackground );
		hBackground = hTempBits;

		DeleteDC( hdcImg );
		SelectObject( hdcMem, hBackground );

		/* set alpha for windows */
		for( x = titleRect.left; x <= titleRect.right; x++ )
			for( y = titleRect.top; y <= titleRect.bottom; y++ ){
				pixel = GetPixel( hdcMem, x, y );
				R = (menuR - GetRValue( pixel )) * alpha + GetRValue( pixel );
				G = (menuG - GetGValue( pixel )) * alpha + GetGValue( pixel );
				B = (menuB - GetBValue( pixel )) * alpha + GetBValue( pixel );
				SetPixel( hdcMem, x, y, RGB(R,G,B) );
			}
		for( x = listRect.left; x <= listRect.right; x++ )
			for( y = listRect.top; y <= listRect.bottom; y++ ){
				pixel = GetPixel( hdcMem, x, y );
				R = (menuR - GetRValue( pixel )) * alpha + GetRValue( pixel );
				G = (menuG - GetGValue( pixel )) * alpha + GetGValue( pixel );
				B = (menuB - GetBValue( pixel )) * alpha + GetBValue( pixel );
				SetPixel( hdcMem, x, y, RGB(R,G,B) );
			}
		for( x = picRect.left; x <= picRect.right; x++ )
			for( y = picRect.top; y <= picRect.bottom; y++ ){
				pixel = GetPixel( hdcMem, x, y );
				R = (menuR - GetRValue( pixel )) * alpha + GetRValue( pixel );
				G = (menuG - GetGValue( pixel )) * alpha + GetGValue( pixel );
				B = (menuB - GetBValue( pixel )) * alpha + GetBValue( pixel );
				SetPixel( hdcMem, x, y, RGB(R,G,B) );
			}
		for( x = descRect.left; x <= descRect.right; x++ )
			for( y = descRect.top; y <= descRect.bottom; y++ ){
				pixel = GetPixel( hdcMem, x, y );
				R = (menuR - GetRValue( pixel )) * alpha + GetRValue( pixel );
				G = (menuG - GetGValue( pixel )) * alpha + GetGValue( pixel );
				B = (menuB - GetBValue( pixel )) * alpha + GetBValue( pixel );
				SetPixel( hdcMem, x, y, RGB(R,G,B) );
			}
		for( x = captionRect.left; x <= captionRect.right; x++ )
			for( y = captionRect.top; y <= captionRect.bottom; y++ ){
				pixel = GetPixel( hdcMem, x, y );
				R = (menuR - GetRValue( pixel )) * alpha + GetRValue( pixel );
				G = (menuG - GetGValue( pixel )) * alpha + GetGValue( pixel );
				B = (menuB - GetBValue( pixel )) * alpha + GetBValue( pixel );
				SetPixel( hdcMem, x, y, RGB(R,G,B) );
			}

		/* windows */
		SetRect( &rect, titleRect.left, titleRect.top, titleRect.right + 1, titleRect.bottom + 1 );
		FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(borderR,borderG,borderB) ) );
		SetRect( &rect, listRect.left, listRect.top, listRect.right + 1, listRect.bottom + 1 );
		FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(borderR,borderG,borderB) ) );
		SetRect( &rect, picRect.left, picRect.top, picRect.right + 1, picRect.bottom + 1 );
		FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(borderR,borderG,borderB) ) );
		SetRect( &rect, descRect.left, descRect.top, descRect.right + 1, descRect.bottom + 1 );
		FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(borderR,borderG,borderB) ) );
		SetRect( &rect, captionRect.left, captionRect.top, captionRect.right + 1, captionRect.bottom + 1 );
		FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(borderR,borderG,borderB) ) );

		/* text metrics */
		font = CreateFont( fontsize, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, ANSI_CHARSET, 
						   OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
						   ANTIALIASED_QUALITY, DEFAULT_PITCH | FF_ROMAN, fontname );
		SelectObject( hdc, font );
		SelectObject( hdcMem, font );
		SetTextColor( hdcMem, RGB(fontR,fontG,fontB) );
		GetTextMetrics( hdc, &tm );

		/* title */
		SetBkMode( hdcMem, TRANSPARENT );
		DrawText( hdcMem, TEXT("GameLoader v1.0"), -1, &titleRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER );

		/* selection frame */
		yListTop = listRect.top + (listRect.bottom - listRect.top) / 2 - tm.tmHeight / 2;
		SetRect( &rect, listRect.left + 5, yListTop - 1, listRect.right - 5, yListTop + tm.tmHeight + 3);
		FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(selectR,selectG,selectB)) );

		/* description window position */
		yDescTop = descRect.top + 1;

		DeleteDC( hdcMem );
		ReleaseDC( hwnd, hdc );
		DestroyWindow( splash );

		return 0;

	case WM_PAINT:
		hdc = GetDC( hwnd );
		hdcMem = CreateCompatibleDC( hdc );
		hdcImg = CreateCompatibleDC( hdc );
		
		hTempBits = CreateCompatibleBitmap( hdc, xClient, yClient );
		SelectObject( hdcMem, hTempBits );

		SetBkMode(hdcMem, TRANSPARENT);
		SetTextColor( hdcMem, RGB(fontR,fontG,fontB) );
		SelectObject( hdcMem, font );

		/* copy background to memory */
		SelectObject( hdcImg, hBackground );
		BitBlt( hdcMem, 0, 0, xClient, yClient, hdcImg, 0, 0, SRCCOPY );
		DeleteDC( hdcImg );
		
		/* paint game list */
		SetRect( &rect, listRect.left + 1, listRect.top + 1, listRect.right, listRect.bottom);
		SelectClipRgn( hdcMem, CreateRectRgnIndirect( &rect ) );
		currentgame = gamehead;
		y = yListTop; 
		while( currentgame ) {
			TextOut( hdcMem, listRect.left + 10, y, currentgame->node.name, strlen(currentgame->node.name) );
			currentgame = currentgame->next;
			y += tm.tmHeight + 3;
		}

		if( selectedGame ) {
			/* paint description */
			if( selectedGame->node.desc[0] ) {
				SetRect( &rect, descRect.left + 1, descRect.top + 1, descRect.right - 21, descRect.bottom );
				SelectClipRgn( hdcMem, CreateRectRgnIndirect( &rect ) );
				SetRect( &rect, descRect.left + 10, yDescTop, descRect.right - 21, descRect.bottom );
				DrawText( hdcMem, selectedGame->node.desc, -1, &rect, DT_EXPANDTABS | DT_WORDBREAK );
			}

			/* paint caption */
			if( selectedGame->node.path[0] ) {
				SetRect( &rect, captionRect.left + 1, captionRect.top + 1, captionRect.right - 1, captionRect.bottom - 1 );
				SelectClipRgn( hdcMem, CreateRectRgnIndirect( &rect ) );
				TextOut( hdcMem, captionRect.left + 10, 
					     captionRect.top + (captionRect.bottom - captionRect.top) / 2 - tm.tmHeight / 2,
						 selectedGame->node.path, strlen(selectedGame->node.path) );
			}
			/* paint screenshot */
			if( selectedGame->node.pic[0] ) {
				SetRect( &rect, picRect.left + 1, picRect.top + 1, picRect.right, picRect.bottom );
				SelectClipRgn( hdcMem, CreateRectRgnIndirect( &rect ) );

				hdcImg = CreateCompatibleDC( hdc );
				DeleteObject( hPicBits );
				hPicBits = read_JPEG_file( selectedGame->node.pic );
				GetObject( hPicBits, sizeof( BITMAP ), &bitmap );
				SelectObject( hdcImg, hPicBits );

				BitBlt( hdcMem, picRect.left + (picRect.right - picRect.left) / 2 - bitmap.bmWidth / 2, 
								picRect.top + (picRect.bottom - picRect.top) / 2 - bitmap.bmHeight / 2, 
								bitmap.bmWidth, bitmap.bmHeight, hdcImg, 0, 0, SRCCOPY );
				
				if( hPicBits ) {
					SetRect( &rect, picRect.left + (picRect.right - picRect.left) / 2 - bitmap.bmWidth / 2,
								    picRect.top + (picRect.bottom - picRect.top) / 2 - bitmap.bmHeight / 2,
									picRect.left + (picRect.right - picRect.left) / 2 - bitmap.bmWidth / 2 + bitmap.bmWidth,
									picRect.top + (picRect.bottom - picRect.top) / 2 - bitmap.bmHeight / 2 + bitmap.bmHeight );
					FrameRect( hdcMem, &rect, CreateSolidBrush( RGB(selectR,selectG,selectB) ) );
				}

				DeleteDC( hdcImg );
			}
		}

		/* paint scroll icons */
/*
		SelectClipRgn( hdcMem, NULL );

		if( focus == LISTWINDOW ) {
			SetRect( &rect, listRect.right - 20, listRect.top, listRect.right + 1, listRect.bottom + 1 );
			FillRect( hdcMem, &rect, CreateSolidBrush(RGB(scrollR,scrollG,scrollB)) );

			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
			SetRect( &rect, listRect.right - 20, listRect.top, listRect.right + 1, listRect.top + 20 );
			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
			SetRect( &rect, listRect.right - 20, listRect.bottom - 20, listRect.right + 1, listRect.bottom + 1 );
			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
		}
		else {
			SetRect( &rect, descRect.right - 20, descRect.top, descRect.right + 1, descRect.bottom + 1 );
			FillRect( hdcMem, &rect, CreateSolidBrush(RGB(scrollR,scrollG,scrollB)) );

			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
			SetRect( &rect, descRect.right - 20, descRect.top, descRect.right + 1, descRect.top + 20 );
			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
			SetRect( &rect, descRect.right - 20, descRect.bottom - 20, descRect.right + 1, descRect.bottom + 1 );
			FrameRect( hdcMem, &rect, CreateSolidBrush(RGB(borderR,borderG,borderB)) );
		}
*/

		/* paint screen */
		BitBlt( hdc, 0, 0, xClient, yClient, hdcMem, 0, 0, SRCCOPY );

		DeleteObject( hTempBits );
		DeleteDC( hdcMem );
		ReleaseDC( hwnd, hdc );

		ValidateRect( hwnd, NULL );

		return 0;

	case WM_ERASEBKGND:
		return 1;

	case WM_KEYDOWN:
		switch( wParam ) {

		case VK_ESCAPE:				/* kill program */
			SendMessage( hwnd, WM_DESTROY, 0, 0 );
			break;

		case VK_RETURN:				/* start game */
			if( selectedGame ) {
				ShellExecute( NULL, TEXT("open"), 
					                GetProgramStr( selectedGame->node.path ),
					                GetParamStr( selectedGame->node.path ), 
									GetDirectoryStr( selectedGame->node.path ), 
									SW_SHOWNORMAL );
			}
			break;

		case VK_LEFT:
			focus = LISTWINDOW;		/* set foucs to list */
			break;

		case VK_RIGHT:
			focus = DESCWINDOW;		/* set focus to description */
			break;

		case VK_UP:					/* scroll up */
			if( selectedGame ) 
				if( focus == LISTWINDOW ) {
					if( selectedGame->prev ) {
						PlaySound( TEXT("img/coin.wav"), NULL, SND_FILENAME | SND_ASYNC );
						yDescTop = descRect.top + 1;
						selectedGame = selectedGame->prev;
						yListTop += tm.tmHeight + 3;
					}
				}
				else {
					yDescTop += tm.tmHeight + 3;
					/* Dont scroll past top */
					if( yDescTop > descRect.top + 1 )
						yDescTop = descRect.top + 1;	
				}

			InvalidateRect( hwnd, NULL, FALSE );
			break;

		case VK_DOWN:				/* scroll down */
			if( selectedGame ) 
				if( focus == LISTWINDOW ) {
					if( selectedGame->next ) {
						PlaySound( TEXT("img/coin.wav"), NULL, SND_FILENAME | SND_ASYNC );
						yDescTop = descRect.top + 1;
						selectedGame = selectedGame->next;
						yListTop -= tm.tmHeight + 3;
					}
				}
				else 
					yDescTop -= tm.tmHeight + 3;
			InvalidateRect( hwnd, NULL, FALSE );				
			break;

		}
		 return 0;

    case WM_DESTROY:
		CleanUp();
		DeleteObject( hBackground );
		DeleteObject( hPicBits );
		DeleteObject( font );

		PostQuitMessage (0) ;
		return 0 ;
    }
    return DefWindowProc (hwnd, message, wParam, lParam) ;
}